--[[

AVDB 0.1.0
 2014 RoLex

This script is free software; You can redistribute
it and modify it under the terms of the GNU General
Public License as published by the Free Software
Foundation, either version 3 of the license, or at
your option any later version.

This script is distributed in the hope that it will
be useful, but without any warranty, without even
the implied warranty of merchantability or fitness
for a particular purpose. See the GNU General Public
License for more details.

Please see http://www.gnu.org/licenses/ for a copy
of the GNU General Public License.

Changelog:

	0.1.0 - Initial release

]]--

local hex_conf = {
	delay = 1800,									-- load delay in seconds
	info = true,									-- information in feed
	skip = 4000,									-- protected access level
	bext = "Virus spreaders are not welcome here",	-- detection ban reason
	bype = 1,										-- detection ban type
	bime = 43200									-- ban time in minutes
}

local hex_data = {
	http = require ("socket.http"),
	ltn12 = require ("ltn12"),
	timer = Hex.Timer (),
	url = "http://te-home.net/avdb.php?do=load",
	tick = 0,
	avdb = {}
}

function OnLoad ()
	Hex.SetAboutString ("2014 RoLex")

	if not hex_data.http then
		print ("hex_avdb.lua: Unable to load module Socket.HTTP")
		return false
	end

	if not hex_data.ltn12 then
		print ("hex_avdb.lua: Unable to load module Socket.LTN12")
		return false
	end

	if not hex_data.timer:Start (1000, hex_conf.delay * 1000, "OnTimer") then
		print ("hex_avdb.lua: Unable to start timer")
		return false
	end

	return true
end

function OnUnload ()
	if not hex_data.timer:Stop () then
		print ("hex_avdb.lua: Unable to stop timer")
	end

	hex_conf, hex_data = nil, nil
end

function OnUserJoined (uid)
	avdbcheck (uid)
end

function OnTimer (tid)
	local data = {}

	local res, err = hex_data.http.request { -- todo: make use of headears
		headers = {
			["User-Agent"] = "HeXHub/" .. Hex._GetVersion ()
		},
		sink = hex_data.ltn12.sink.table (data),
		url = hex_data.url .. "&time=" .. tostring (hex_data.tick) .. "&cotime=0&nosort=1"
	}

	hex_data.tick = os.time ()

	if res and err and err == 200 then
		local total, count = 0, 0

		if data and type (data) == "table" then
			data = table.concat (data)
		end

		if data and type (data) == "string" and # data > 0 then
			for line in data:gmatch ("[^\r\n]+") do
				local _, _, nick, addr, size = line:find ("^([^ ]+)|(%d+%.%d+%.%d+%.%d+)|(%d+)$")

				if nick and addr and size then
					size = tonumber (size)
					local ok = true

					for id, item in pairs (hex_data.avdb) do
						if item ["nick"] == nick and item ["addr"] == addr then
							if item ["size"] ~= size then
								hex_data.avdb [id]["size"] = size
								count = count + 1
							end

							ok = false
							break
						end
					end

					if ok then
						table.insert (hex_data.avdb, {
							["nick"] = nick,
							["addr"] = addr,
							["size"] = size
						})

						count = count + 1
					end

					total = total + 1
				end
			end
		end

		if hex_conf.info then
			Hex.OpchatMessage (string.format ("Loaded %d of %d items: AVDB", count, total))
		end

		if count > 0 then
			local user, ok = Hex.User (-1), true

			while ok do
				avdbcheck (user:GetUserID ())
				ok = user:Next ()
			end
		end
	else
		print ("hex_avdb.lua: Unable to connect to AVDB")
	end
end

function avdbcheck (uid)
	local user = Hex.User (uid)

	if user and user:GetAccessLevel () < hex_conf.skip then
		local size = tonumber (user:GetShareString ()) or 0

		if size > 0 then
			local nick = user:GetClientNick ()
			local addr = user:GetIPString ()

			for _, data in pairs (hex_data.avdb) do
				if (nick == data ["nick"] and addr == data ["addr"]) or (nick == data ["nick"] and size == data ["size"]) or (addr == data ["addr"] and size == data ["size"]) then
					local share = string.format ("%.2f B", size)

					if size >= 1152921504606846976 then
						share = string.format ("%.2f EB", size / 1152921504606846976)
					elseif size >= 1125899906842624 then
						share = string.format ("%.2f PB", size / 1125899906842624)
					elseif size >= 1099511627776 then
						share = string.format ("%.2f TB", size / 1099511627776)
					elseif size >= 1073741824 then
						share = string.format ("%.2f GB", size / 1073741824)
					elseif size >= 1048576 then
						share = string.format ("%.2f MB", size / 1048576)
					elseif size >= 1024 then
						share = string.format ("%.2f KB", size / 1024)
					end

					Hex.OpchatMessage (string.format ("Infected user detected with IP %s and share %s: %s", addr, share, nick))
					avdbreport (nick, addr, size)
					user:BanNickIP (hex_conf.bext .. " _ban" .. tostring (hex_conf.bype) .. "_" .. tostring (hex_conf.bime) .. "m", hex_conf.bime * 60)
					user:SendData ("<" .. Hex.User (0):GetNick () .. "> " .. hex_conf.bext .. "|")
					user:Disconnect ()
					break
				end
			end
		end
	end
end

function avdbreport (nick, addr, size)
	-- todo
end

-- end of file